# Copyright (c) 2014 Hoang Do, Phuc Vo, P. Michiardi, D. Venzano
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg

from sahara.plugins import exceptions as ex
from sahara_plugin_storm.i18n import _


CONF = cfg.CONF


def get_config_value(service, name, cluster=None):
    if cluster:
        for ng in cluster.node_groups:
            if (ng.configuration().get(service) and
                    ng.configuration()[service].get(name)):
                return ng.configuration()[service][name]

    raise ex.ConfigurationError(
        _("Unable to get parameter '%(param_name)s' from "
          "service %(service)s") % {'param_name': name, 'service': service})


def get_plugin_configs():
    return {}


def generate_storm_config(master_hostname, zk_hostnames, version):

    host_cfg = 'nimbus.seeds'
    master_value = [master_hostname.encode('ascii', 'ignore')]

    cfg = {
        host_cfg: master_value,
        "worker.childopts": "-Xmx768m -Djava.net.preferIPv4Stack=true",
        "nimbus.childopts": "-Xmx1024m -Djava.net.preferIPv4Stack=true",
        "supervisor.childopts": "-Djava.net.preferIPv4Stack=true",
        "storm.zookeeper.servers": [i.encode('ascii', 'ignore')
                                    for i in zk_hostnames],
        "ui.childopts": "-Xmx768m -Djava.net.preferIPv4Stack=true",
        "storm.local.dir": "/app/storm"
    }

    # Since pyleus is built using previous versions of Storm we need this
    # option to allow the cluster to be compatible with pyleus topologies as
    #  well as with topologies built using older versions of Storm
    cfg['client.jartransformer.class'] = (
        "org.apache.storm.hack.StormShadeTransformer")

    return cfg


def generate_pyleus_config():
    separator = "\n"
    conf = ("[storm]",
            "storm_cmd_path: /usr/local/storm/bin/storm")

    return separator.join(conf)


def generate_slave_supervisor_conf():
    separator = "\n"
    conf = ("[program:storm-supervisor]",
            'command=bash -exec "cd /usr/local/storm && bin/storm supervisor"',
            "user=storm",
            "autostart=true",
            "autorestart=true",
            "startsecs=10",
            "startretries=999",
            "log_stdout=true",
            "log_stderr=true",
            "logfile=/var/log/storm/supervisor.out",
            "logfile_maxbytes=20MB",
            "logfile_backups=10")

    return separator.join(conf)


def generate_master_supervisor_conf():
    separator = "\n"
    seq_n = ("[program:storm-nimbus]",
             "command=/usr/local/storm/bin/storm nimbus",
             "user=storm",
             "autostart=true",
             "autorestart=true",
             "startsecs=10",
             "startretries=999",
             "log_stdout=true",
             "log_stderr=true",
             "logfile=/var/log/storm/supervisor.out",
             "logfile_maxbytes=20MB",
             "logfile_backups=10")

    seq_u = ("[program:storm-ui]",
             "command=/usr/local/storm/bin/storm ui",
             "user=storm",
             "autostart=true",
             "autorestart=true",
             "startsecs=10",
             "startretries=999",
             "log_stdout=true",
             "log_stderr=true",
             "logfile=/var/log/storm/ui.out",
             "logfile_maxbytes=20MB",
             "logfile_backups=10")

    conf_n = separator.join(seq_n)
    conf_u = separator.join(seq_u)
    conf = (conf_n, conf_u)

    return separator.join(conf)


def generate_zookeeper_conf():
    separator = "\n"
    conf = ("tickTime=2000",
            "dataDir=/var/zookeeper",
            "clientPort=2181")

    return separator.join(conf)


def generate_storm_setup_script(env_configs):
    separator = "\n"
    script_lines = ["#!/bin/bash -x"]
    script_lines.append("echo -n > /usr/local/storm/conf/storm.yaml")
    for line in env_configs:
        script_lines.append('echo "%s" >> /usr/local/storm/conf/storm.yaml'
                            % line)

    return separator.join(script_lines)


def extract_name_values(configs):
    return {cfg['name']: cfg['value'] for cfg in configs}


def _set_config(cfg, gen_cfg, name=None):
    if name in gen_cfg:
        cfg.update(gen_cfg[name]['conf'])
    if name is None:
        for name in gen_cfg:
            cfg.update(gen_cfg[name]['conf'])
    return cfg



# Copyright (c) 2014 Hoang Do, Phuc Vo, P. Michiardi, D. Venzano
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


def start_zookeeper(remote):
    remote.execute_command("sudo %s %s" % (
        "/opt/zookeeper/zookeeper/bin/zkServer.sh",
        "start"))


def start_storm_supervisor(node):
    _create_supervisor_log_file(node)
    _stop_supervisor_deamon(node)
    _start_supervisor_deamon(node)


def start_storm_nimbus_and_ui(node):
    _create_supervisor_log_file(node)
    _stop_supervisor_deamon(node)
    _start_supervisor_deamon(node)


def stop_storm_nimbus_and_ui(node):
    _stop_supervisor_deamon(node)


def stop_storm_supervisor(node):
    _stop_supervisor_deamon(node)


def _start_supervisor_deamon(node):
    node.execute_command("sudo service supervisor start")


def _stop_supervisor_deamon(node):
    node.execute_command("sudo service supervisor stop")


def _create_supervisor_log_file(node):
    node.execute_command("sudo mkdir -p /var/log/storm")
    node.execute_command("sudo chmod -R 777 /var/log/storm")
    node.execute_command("sudo chown -R storm:storm /var/log/storm")



# Copyright (c) 2015 TellesNobrega
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from sahara.plugins import base as pb
from sahara.plugins import conductor
from sahara.plugins import context
from sahara.plugins import edp
from sahara.plugins import exceptions as ex
from sahara_plugin_storm.plugins.storm import plugin as pl
from sahara_plugin_storm.tests.unit import base


class StormPluginTest(base.SaharaWithDbTestCase):
    def setUp(self):
        super(StormPluginTest, self).setUp()
        self.override_config("plugins", ["storm"])
        self.master_host = "master"
        self.master_inst = "6789"
        self.storm_topology_name = 'topology1'
        pb.setup_plugins()

    def _make_master_instance(self, return_code=0):
        master = mock.Mock()
        master.execute_command.return_value = (return_code,
                                               self.storm_topology_name)
        master.hostname.return_value = self.master_host
        master.id = self.master_inst
        return master

    def _get_cluster(self, name, version):
        cluster_dict = {
            'name': name,
            'plugin_name': 'storm',
            'hadoop_version': version,
            'node_groups': []}
        return cluster_dict

    def test_validate_existing_ng_scaling(self):
        data = [
            {'name': 'master',
             'flavor_id': '42',
             'count': 1,
             'node_processes': ['nimbus']},
            {'name': 'slave',
             'flavor_id': '42',
             'count': 1,
             'node_processes': ['supervisor']},
            {'name': 'zookeeper',
             'flavor_id': '42',
             'count': 1,
             'node_processes': ['zookeeper']}
        ]

        cluster_data_101 = self._get_cluster('cluster_1.0.1', '1.0.1')
        cluster_data_110 = self._get_cluster('cluster_1.1.0', '1.1.0')
        cluster_data_120 = self._get_cluster('cluster_1.2.0', '1.2')
        cluster_data_101['node_groups'] = data
        cluster_data_110['node_groups'] = data
        cluster_data_120['node_groups'] = data

        clusters = [cluster_data_101, cluster_data_110, cluster_data_120]

        for cluster_data in clusters:
            cluster = conductor.cluster_create(context.ctx(), cluster_data)
            plugin = pb.PLUGINS.get_plugin(cluster.plugin_name)
            supervisor_id = [node.id for node in cluster.node_groups
                             if node.name == 'supervisor']
            self.assertIsNone(
                plugin._validate_existing_ng_scaling(cluster,
                                                     supervisor_id))

    @mock.patch("sahara_plugin_storm.plugins.storm.plugin.utils")
    def test_validate(self, mock_utils):

        cluster_data = self._get_cluster('cluster', '1.1.0')
        cluster = conductor.cluster_create(context.ctx(), cluster_data)
        plugin = pb.PLUGINS.get_plugin(cluster.plugin_name)

        # number of nimbus nodes != 1 should raise an exception
        fake_ng = mock.Mock()
        fake_ng.count = 0
        mock_ng = mock.Mock(return_value=[fake_ng])
        mock_utils.get_node_groups = mock_ng

        self.assertRaises(ex.RequiredServiceMissingException,
                          plugin.validate, cluster)

        mock_ng.assert_called_once_with(cluster, "nimbus")

        fake_ng.count = 2
        self.assertRaises(ex.InvalidComponentCountException, plugin.validate,
                          cluster)

        mock_ng.assert_called_with(cluster, "nimbus")
        self.assertEqual(2, mock_ng.call_count)

        # no supervisor should raise an exception
        fake_nimbus = mock.Mock()
        fake_nimbus.count = 1

        fake_supervisor = mock.Mock()
        fake_supervisor.count = 0

        mock_ng = mock.Mock(side_effect=[[fake_nimbus], [fake_supervisor]])
        mock_utils.get_node_groups = mock_ng

        self.assertRaises(ex.InvalidComponentCountException, plugin.validate,
                          cluster)

        mock_ng.assert_any_call(cluster, "nimbus")
        mock_ng.assert_any_call(cluster, "supervisor")
        self.assertEqual(2, mock_ng.call_count)

        # one nimbus and one or more supervisors should not raise an exception
        fake_nimbus.count = 1
        fake_supervisor.count = 2

        mock_ng = mock.Mock(side_effect=[[fake_nimbus], [fake_supervisor]])
        mock_utils.get_node_groups = mock_ng

        plugin.validate(cluster)

        mock_ng.assert_any_call(cluster, "nimbus")
        mock_ng.assert_any_call(cluster, "supervisor")
        self.assertEqual(2, mock_ng.call_count)

    def test_validate_additional_ng_scaling(self):
        data = [
            {'name': 'master',
             'flavor_id': '42',
             'count': 1,
             'node_processes': ['nimbus']},
            {'name': 'slave',
             'flavor_id': '42',
             'count': 1,
             'node_processes': ['supervisor']},
            {'name': 'zookeeper',
             'flavor_id': '42',
             'count': 1,
             'node_processes': ['zookeeper']},
            {'name': 'slave2',
             'flavor_id': '42',
             'count': 0,
             'node_processes': ['supervisor']}
        ]

        cluster_data_101 = self._get_cluster('cluster_1.0.1', '1.0.1')
        cluster_data_110 = self._get_cluster('cluster_1.1.0', '1.1.0')
        cluster_data_120 = self._get_cluster('cluster_1.2.0', '1.2')
        cluster_data_101['node_groups'] = data
        cluster_data_110['node_groups'] = data
        cluster_data_120['node_groups'] = data

        clusters = [cluster_data_101, cluster_data_110, cluster_data_120]

        for cluster_data in clusters:
            cluster = conductor.cluster_create(context.ctx(), cluster_data)
            plugin = pb.PLUGINS.get_plugin(cluster.plugin_name)
            supervisor_id = [node.id for node in cluster.node_groups
                             if node.name == 'supervisor']
            self.assertIsNone(
                plugin._validate_additional_ng_scaling(cluster,
                                                       supervisor_id))

    def test_validate_existing_ng_scaling_raises(self):
        data = [
            {'name': 'master',
             'flavor_id': '42',
             'count': 1,
             'node_processes': ['nimbus']},
            {'name': 'slave',
             'flavor_id': '42',
             'count': 1,
             'node_processes': ['supervisor']},
            {'name': 'zookeeper',
             'flavor_id': '42',
             'count': 1,
             'node_processes': ['zookeeper']}
        ]

        cluster_data_101 = self._get_cluster('cluster_1.0.1', '1.0.1')
        cluster_data_110 = self._get_cluster('cluster_1.1.0', '1.1.0')
        cluster_data_120 = self._get_cluster('cluster_1.2.0', '1.2')
        cluster_data_101['node_groups'] = data
        cluster_data_110['node_groups'] = data
        cluster_data_120['node_groups'] = data

        clusters = [cluster_data_101, cluster_data_110, cluster_data_120]

        for cluster_data in clusters:
            cluster = conductor.cluster_create(context.ctx(), cluster_data)
            plugin = pb.PLUGINS.get_plugin(cluster.plugin_name)
            master_id = [node.id for node in cluster.node_groups
                         if node.name == 'master']
            self.assertRaises(ex.NodeGroupCannotBeScaled,
                              plugin._validate_existing_ng_scaling,
                              cluster, master_id)

    def test_validate_additional_ng_scaling_raises(self):
        data = [
            {'name': 'master',
             'flavor_id': '42',
             'count': 1,
             'node_processes': ['nimbus']},
            {'name': 'slave',
             'flavor_id': '42',
             'count': 1,
             'node_processes': ['supervisor']},
            {'name': 'zookeeper',
             'flavor_id': '42',
             'count': 1,
             'node_processes': ['zookeeper']},
            {'name': 'master2',
             'flavor_id': '42',
             'count': 0,
             'node_processes': ['nimbus']}
        ]

        cluster_data_101 = self._get_cluster('cluster_1.0.1', '1.0.1')
        cluster_data_110 = self._get_cluster('cluster_1.1.0', '1.1.0')
        cluster_data_120 = self._get_cluster('cluster_1.2.0', '1.2')
        cluster_data_101['node_groups'] = data
        cluster_data_110['node_groups'] = data
        cluster_data_120['node_groups'] = data

        clusters = [cluster_data_101, cluster_data_110, cluster_data_120]

        for cluster_data in clusters:
            cluster = conductor.cluster_create(context.ctx(), cluster_data)
            plugin = pb.PLUGINS.get_plugin(cluster.plugin_name)
            master_id = [node.id for node in cluster.node_groups
                         if node.name == 'master2']
            self.assertRaises(ex.NodeGroupCannotBeScaled,
                              plugin._validate_existing_ng_scaling,
                              cluster, master_id)

    def test_get_open_port(self):
        plugin_storm = pl.StormProvider()
        cluster = mock.Mock()
        ng = mock.Mock()
        ng.node_processes = ['nimbus']
        cluster.node_groups = [ng]
        ng.cluster = cluster
        ports = plugin_storm.get_open_ports(ng)
        self.assertEqual([8080], ports)

    def _test_engine(self, version, job_type, eng):
        cluster_dict = self._get_cluster('demo', version)

        cluster = conductor.cluster_create(context.ctx(), cluster_dict)
        plugin = pb.PLUGINS.get_plugin(cluster.plugin_name)
        self.assertIsInstance(plugin.get_edp_engine(cluster, job_type), eng)

    def test_plugin101_edp_storm_engine(self):
        self._test_engine('1.0.1', edp.JOB_TYPE_STORM,
                          edp.PluginsStormJobEngine)

    def test_plugin101_edp_storm_pyleus_engine(self):
        self._test_engine('1.0.1', edp.JOB_TYPE_PYLEUS,
                          edp.PluginsStormPyleusJobEngine)

    def test_plugin110_edp_storm_engine(self):
        self._test_engine('1.1.0', edp.JOB_TYPE_STORM,
                          edp.PluginsStormJobEngine)

    def test_plugin110_edp_storm_pyleus_engine(self):
        self._test_engine('1.1.0', edp.JOB_TYPE_PYLEUS,
                          edp.PluginsStormPyleusJobEngine)

    def test_plugin120_edp_storm_engine(self):
        self._test_engine('1.2', edp.JOB_TYPE_STORM,
                          edp.PluginsStormJobEngine)

    def test_plugin120_edp_storm_pyleus_engine(self):
        self._test_engine('1.2', edp.JOB_TYPE_PYLEUS,
                          edp.PluginsStormPyleusJobEngine)



# Copyright (c) 2014 Hoang Do, Phuc Vo, P. Michiardi, D. Venzano
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

from oslo_log import log as logging
import six
import yaml

from sahara.plugins import conductor
from sahara.plugins import context
from sahara.plugins import exceptions as ex
from sahara.plugins import provisioning as p
from sahara.plugins import utils
from sahara_plugin_storm.i18n import _
from sahara_plugin_storm.plugins.storm import config_helper as c_helper
from sahara_plugin_storm.plugins.storm import edp_engine
from sahara_plugin_storm.plugins.storm import run_scripts as run

LOG = logging.getLogger(__name__)


class StormProvider(p.ProvisioningPluginBase):
    def __init__(self):
        self.processes = {
            "Zookeeper": ["zookeeper"],
            "Storm": ["nimbus", "supervisor"]
        }

    def get_title(self):
        return "Apache Storm"

    def get_description(self):
        return (
            _("This plugin provides an ability to launch Storm "
              "cluster without any management consoles."))

    def get_labels(self):
        default = {'enabled': {'status': True}, 'stable': {'status': True}}
        deprecated = {'enabled': {'status': True},
                      'deprecated': {'status': True}}
        result = {'plugin_labels': copy.deepcopy(default)}
        result['version_labels'] = {
            '1.2': copy.deepcopy(default),
            '1.1.0': copy.deepcopy(default),
            '1.0.1': copy.deepcopy(deprecated),
        }
        return result

    def get_versions(self):
        return ['1.0.1', '1.1.0', '1.2']

    def get_configs(self, storm_version):
        return c_helper.get_plugin_configs()

    def get_node_processes(self, storm_version):
        return self.processes

    def validate(self, cluster):
        # validate Storm Master Node and Storm Slaves
        sm_count = sum([ng.count for ng
                        in utils.get_node_groups(cluster, "nimbus")])

        if sm_count < 1:
            raise ex.RequiredServiceMissingException("Storm nimbus")

        if sm_count >= 2:
            raise ex.InvalidComponentCountException("Storm nimbus", "1",
                                                    sm_count)

        sl_count = sum([ng.count for ng
                        in utils.get_node_groups(cluster, "supervisor")])

        if sl_count < 1:
            raise ex.InvalidComponentCountException("Storm supervisor",
                                                    _("1 or more"),
                                                    sl_count)

    def update_infra(self, cluster):
        pass

    def configure_cluster(self, cluster):
        self._setup_instances(cluster)

    def start_cluster(self, cluster):
        sm_instance = utils.get_instance(cluster, "nimbus")
        sl_instances = utils.get_instances(cluster, "supervisor")
        zk_instances = utils.get_instances(cluster, "zookeeper")

        # start zookeeper processes
        self._start_zookeeper_processes(zk_instances)

        # start storm master
        if sm_instance:
            self._start_storm_master(sm_instance)

        # start storm slaves
        self._start_slave_processes(sl_instances)

        LOG.info("Cluster {cluster} has been started successfully".format(
            cluster=cluster.name))
        self._set_cluster_info(cluster)

    def get_edp_engine(self, cluster, job_type):
        if job_type in edp_engine.EdpStormEngine.get_supported_job_types():
            return edp_engine.EdpStormEngine(cluster)
        if job_type in edp_engine.EdpPyleusEngine.get_supported_job_types():
            return edp_engine.EdpPyleusEngine(cluster)
        return None

    def get_edp_job_types(self, versions=None):
        res = {}
        for vers in self.get_versions():
            if not versions or vers in versions:
                storm_engine = edp_engine.EdpStormEngine
                pyleus_engine = edp_engine.EdpPyleusEngine
                res[vers] = (storm_engine.get_supported_job_types() +
                             pyleus_engine.get_supported_job_types())
        return res

    def get_edp_config_hints(self, job_type, version):
        if edp_engine.EdpStormEngine.edp_supported(version):
            return edp_engine.EdpStormEngine.get_possible_job_config(job_type)
        if edp_engine.EdpPyleusEngine.edp_supported(version):
            return edp_engine.EdpPyleusEngine.get_possible_job_config(job_type)
        return {}

    def get_open_ports(self, node_group):
        ports_map = {
            'nimbus': [8080]
        }

        ports = []
        for process in node_group.node_processes:
            if process in ports_map:
                ports.extend(ports_map[process])

        return ports

    def _extract_configs_to_extra(self, cluster):
        st_master = utils.get_instance(cluster, "nimbus")
        zk_servers = utils.get_instances(cluster, "zookeeper")

        extra = dict()

        config_instances = ''
        if st_master is not None:
            if zk_servers is not None:
                zknames = []
                for zk in zk_servers:
                    zknames.append(zk.hostname())

            config_instances = c_helper.generate_storm_config(
                st_master.hostname(),
                zknames,
                cluster.hadoop_version)

        config = self._convert_dict_to_yaml(config_instances)
        supervisor_conf = c_helper.generate_slave_supervisor_conf()
        nimbus_ui_conf = c_helper.generate_master_supervisor_conf()
        zk_conf = c_helper.generate_zookeeper_conf()
        pyleus_conf = c_helper.generate_pyleus_config()

        for ng in cluster.node_groups:
            extra[ng.id] = {
                'st_instances': config,
                'slave_sv_conf': supervisor_conf,
                'master_sv_conf': nimbus_ui_conf,
                'zk_conf': zk_conf,
                'pyleus_conf': pyleus_conf
            }

        return extra

    @utils.event_wrapper(
        True, step=utils.start_process_event_message("StormMaster"))
    def _start_storm_master(self, sm_instance):
        with utils.get_remote(sm_instance) as r:
            run.start_storm_nimbus_and_ui(r)
            LOG.info("Storm master at {host} has been started".format(
                host=sm_instance.hostname()))

    def _start_slave_processes(self, sl_instances):
        if len(sl_instances) == 0:
            return
        utils.add_provisioning_step(
            sl_instances[0].cluster_id,
            utils.start_process_event_message("Slave"), len(sl_instances))

        with context.PluginsThreadGroup() as tg:
            for i in sl_instances:
                tg.spawn('storm-start-sl-%s' % i.instance_name,
                         self._start_slaves, i)

    @utils.event_wrapper(True)
    def _start_slaves(self, instance):
        with instance.remote() as r:
            run.start_storm_supervisor(r)

    def _start_zookeeper_processes(self, zk_instances):
        if len(zk_instances) == 0:
            return

        utils.add_provisioning_step(
            zk_instances[0].cluster_id,
            utils.start_process_event_message("Zookeeper"), len(zk_instances))

        with context.PluginsThreadGroup() as tg:
            for i in zk_instances:
                tg.spawn('storm-start-zk-%s' % i.instance_name,
                         self._start_zookeeper, i)

    @utils.event_wrapper(True)
    def _start_zookeeper(self, instance):
        with instance.remote() as r:
            run.start_zookeeper(r)

    def _setup_instances(self, cluster, instances=None):
        extra = self._extract_configs_to_extra(cluster)

        if instances is None:
            instances = utils.get_instances(cluster)

        self._push_configs_to_nodes(cluster, extra, instances)

    def _push_configs_to_nodes(self, cluster, extra, new_instances):
        all_instances = utils.get_instances(cluster)
        utils.add_provisioning_step(
            cluster.id, _("Push configs to nodes"), len(all_instances))

        with context.PluginsThreadGroup() as tg:
            for instance in all_instances:
                if instance in new_instances:
                    tg.spawn('storm-configure-%s' % instance.instance_name,
                             self._push_configs_to_new_node, cluster,
                             extra, instance)
                else:
                    tg.spawn('storm-reconfigure-%s' % instance.instance_name,
                             self._push_configs_to_existing_node, cluster,
                             extra, instance)

    def _convert_dict_to_yaml(self, dict_to_convert):
        new_dict = dict_to_convert.copy()
        for key in dict_to_convert:
            if isinstance(dict_to_convert[key], six.string_types):
                new_dict[key] = "\"" + dict_to_convert[key] + "\""

        stream = yaml.dump(new_dict, default_flow_style=False)
        stream = stream.replace("\'", "")

        return stream

    @utils.event_wrapper(True)
    def _push_configs_to_new_node(self, cluster, extra, instance):
        ng_extra = extra[instance.node_group.id]

        files_supervisor = {
            '/etc/supervisor/supervisord.conf': ng_extra['slave_sv_conf']
        }
        files_storm = {
            '/usr/local/storm/conf/storm.yaml': ng_extra['st_instances']
        }
        files_zk = {
            '/opt/zookeeper/zookeeper/conf/zoo.cfg': ng_extra['zk_conf']
        }
        files_supervisor_master = {
            '/etc/supervisor/supervisord.conf': ng_extra['master_sv_conf']
        }
        file_pyleus_conf = {
            '/home/ubuntu/.pyleus.conf': ng_extra['pyleus_conf']
        }

        with utils.get_remote(instance) as r:
            node_processes = instance.node_group.node_processes
            r.write_files_to(files_storm, run_as_root=True)
            if 'zookeeper' in node_processes:
                self._push_zk_configs(r, files_zk)
            if 'nimbus' in node_processes:
                self._push_supervisor_configs(r, files_supervisor_master)
                self._push_supervisor_configs(r, file_pyleus_conf)
            if 'supervisor' in node_processes:
                self._push_supervisor_configs(r, files_supervisor)

    @utils.event_wrapper(True)
    def _push_configs_to_existing_node(self, cluster, extra, instance):
        node_processes = instance.node_group.node_processes
        need_storm_update = ('nimbus' in node_processes or
                             'supervisor' in node_processes)
        need_zookeeper_update = 'zookeeper' in node_processes

        ng_extra = extra[instance.node_group.id]
        r = utils.get_remote(instance)

        if need_storm_update:
            storm_path = '/usr/local/storm/conf/storm.yaml'
            files_storm = {storm_path: ng_extra['st_instances']}
            r.write_files_to(files_storm)

        if need_zookeeper_update:
            zk_path = '/opt/zookeeper/zookeeper/conf/zoo.cfg'
            files_zookeeper = {zk_path: ng_extra['zk_conf']}
            self._push_zk_configs(r, files_zookeeper)

    def _set_cluster_info(self, cluster):
        st_master = utils.get_instance(cluster, "nimbus")
        info = {}

        if st_master:
            port = "8080"

            info['Storm'] = {
                'Web UI': 'http://%s:%s' % (
                    st_master.get_ip_or_dns_name(), port)
            }
        ctx = context.ctx()
        conductor.cluster_update(ctx, cluster, {'info': info})

    def _push_zk_configs(self, r, files):
        r.write_files_to(files, run_as_root=True)

    def _push_supervisor_configs(self, r, files):
        r.append_to_files(files, run_as_root=True)

    # Scaling

    def _get_running_topologies_names(self, cluster):
        master = utils.get_instance(cluster, "nimbus")

        cmd = (
            "%(storm)s -c nimbus.host=%(host)s "
            "list | grep ACTIVE | awk '{print $1}'") % (
            {
                "storm": "/usr/local/storm/bin/storm",
                "host": master.hostname()
            })

        with utils.get_remote(master) as r:
            ret, stdout = r.execute_command(cmd)
        names = stdout.split('\n')
        topology_names = names[0:len(names)-1]
        return topology_names

    @utils.event_wrapper(True, step=_("Rebalance Topology"),
                         param=('cluster', 1))
    def rebalance_topology(self, cluster):
        topology_names = self._get_running_topologies_names(cluster)
        master = utils.get_instance(cluster, "nimbus")

        for topology_name in topology_names:
            cmd = (
                '%(rebalance)s -c nimbus.host=%(host)s %(topology_name)s') % (
                {
                    "rebalance": "/usr/local/storm/bin/storm rebalance",
                    "host": master.hostname(),
                    "topology_name": topology_name
                })

            with utils.get_remote(master) as r:
                ret, stdout = r.execute_command(cmd)

    def validate_scaling(self, cluster, existing, additional):
        self._validate_existing_ng_scaling(cluster, existing)
        self._validate_additional_ng_scaling(cluster, additional)

    def scale_cluster(self, cluster, instances):
        self._setup_instances(cluster, instances)
        # start storm slaves
        self._start_slave_processes(instances)
        self.rebalance_topology(cluster)
        LOG.info("Storm scaling has been started.")

    def _get_scalable_processes(self):
        return ["supervisor"]

    def _validate_additional_ng_scaling(self, cluster, additional):
        scalable_processes = self._get_scalable_processes()

        for ng_id in additional:
            ng = utils.get_by_id(cluster.node_groups, ng_id)
            if not set(ng.node_processes).issubset(scalable_processes):
                raise ex.NodeGroupCannotBeScaled(
                    ng.name, _("Storm plugin cannot scale nodegroup"
                               " with processes: %s") %
                    ' '.join(ng.node_processes))

    def _validate_existing_ng_scaling(self, cluster, existing):
        scalable_processes = self._get_scalable_processes()
        for ng in cluster.node_groups:
            if ng.id in existing:
                if not set(ng.node_processes).issubset(scalable_processes):
                    raise ex.NodeGroupCannotBeScaled(
                        ng.name, _("Storm plugin cannot scale nodegroup"
                                   " with processes: %s") %
                        ' '.join(ng.node_processes))



# Copyright (c) 2015 Telles Nobrega.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara.plugins import edp
from sahara.plugins import exceptions as ex
from sahara_plugin_storm.i18n import _


class EdpStormEngine(edp.PluginsStormJobEngine):

    edp_base_version = "1.0.1"

    @staticmethod
    def edp_supported(version):
        return version >= EdpStormEngine.edp_base_version

    def validate_job_execution(self, cluster, job, data):
        if not self.edp_supported(cluster.hadoop_version):
            raise ex.PluginInvalidDataException(
                _('Storm {base} required to run {type} jobs').format(
                    base=EdpStormEngine.edp_base_version, type=job.type))

        super(EdpStormEngine, self).validate_job_execution(cluster, job, data)


class EdpPyleusEngine(edp.PluginsStormPyleusJobEngine):

    edp_base_version = "1.0.1"

    @staticmethod
    def edp_supported(version):
        return version >= EdpPyleusEngine.edp_base_version

    def validate_job_execution(self, cluster, job, data):
        if not self.edp_supported(cluster.hadoop_version):
            raise ex.PluginInvalidDataException(
                _('Storm {base} required to run {type} jobs').format(
                    base=EdpPyleusEngine.edp_base_version, type=job.type))

        super(EdpPyleusEngine, self).validate_job_execution(cluster, job, data)



#   Copyright 2017 Massachusetts Open Cloud
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

from testtools import testcase

from sahara_plugin_storm.plugins.storm import config_helper as s_config
from sahara_plugin_storm.plugins.storm import plugin as s_plugin


class TestStormConfigHelper(testcase.TestCase):

    def test_generate_storm_config(self):
        STORM_101 = '1.0.1'
        STORM_110 = '1.1.0'
        STORM_120 = '1.2'
        tested_versions = []
        master_hostname = "s-master"
        zk_hostnames = ["s-zoo"]
        configs_101 = s_config.generate_storm_config(
            master_hostname, zk_hostnames, STORM_101)
        self.assertNotIn('nimbus.host', configs_101.keys())
        self.assertIn('nimbus.seeds', configs_101.keys())
        self.assertIn('client.jartransformer.class', configs_101.keys())
        self.assertEqual(configs_101['client.jartransformer.class'],
                         'org.apache.storm.hack.StormShadeTransformer')
        tested_versions.append(STORM_101)
        configs_110 = s_config.generate_storm_config(
            master_hostname, zk_hostnames, STORM_110)
        self.assertNotIn('nimbus.host', configs_110.keys())
        self.assertIn('nimbus.seeds', configs_110.keys())
        self.assertIn('client.jartransformer.class', configs_110.keys())
        self.assertEqual(configs_110['client.jartransformer.class'],
                         'org.apache.storm.hack.StormShadeTransformer')
        tested_versions.append(STORM_110)
        configs_120 = s_config.generate_storm_config(
            master_hostname, zk_hostnames, STORM_120)
        self.assertNotIn('nimbus.host', configs_120.keys())
        self.assertIn('nimbus.seeds', configs_120.keys())
        self.assertIn('client.jartransformer.class', configs_120.keys())
        self.assertEqual(configs_120['client.jartransformer.class'],
                         'org.apache.storm.hack.StormShadeTransformer')
        tested_versions.append(STORM_120)
        storm = s_plugin.StormProvider()
        self.assertEqual(storm.get_versions(), tested_versions)
